@@ -37,6 +37,18 @@
 
 #include <string>
 
+#ifndef HAVE_STRNLEN
+static size_t strnlen (const char *s, size_t len)
+{
+    for (size_t i = 0; i < len; i++) {
+        if (s[i] == '\0')
+            return i + 1;
+    }
+
+    return len;
+}
+#endif
+
 zmq::ipc_address_t::ipc_address_t ()
 {
     memset (&_address, 0, sizeof _address);
