@@ -58,64 +58,23 @@ zmq::socks_connecter_t::socks_connecter_t (class io_thread_t *io_thread_,
                                            address_t *addr_,
                                            address_t *proxy_addr_,
                                            bool delayed_start_) :
-    own_t (io_thread_, options_),
-    io_object_t (io_thread_),
-    _addr (addr_),
+    stream_connecter_base_t (
+      io_thread_, session_, options_, addr_, delayed_start_),
     _proxy_addr (proxy_addr_),
-    _status (unplugged),
-    _s (retired_fd),
-    _handle (static_cast<handle_t> (NULL)),
-    _handle_valid (false),
-    _delayed_start (delayed_start_),
-    _timer_started (false),
-    _session (session_),
-    _current_reconnect_ivl (options.reconnect_ivl)
+    _status (unplugged)
 {
-    zmq_assert (_addr);
-    zmq_assert (_addr->protocol == "tcp");
+    zmq_assert (_addr->protocol == protocol_name::tcp);
     _proxy_addr->to_string (_endpoint);
-    _socket = _session->get_socket ();
 }
 
 zmq::socks_connecter_t::~socks_connecter_t ()
 {
-    zmq_assert (_s == retired_fd);
     LIBZMQ_DELETE (_proxy_addr);
 }
 
-void zmq::socks_connecter_t::process_plug ()
-{
-    if (_delayed_start)
-        start_timer ();
-    else
-        initiate_connect ();
-}
-
-void zmq::socks_connecter_t::process_term (int linger_)
-{
-    switch (_status) {
-        case unplugged:
-            break;
-        case waiting_for_reconnect_time:
-            cancel_timer (reconnect_timer_id);
-            break;
-        case waiting_for_proxy_connection:
-        case sending_greeting:
-        case waiting_for_choice:
-        case sending_request:
-        case waiting_for_response:
-            rm_fd (_handle);
-            if (_s != retired_fd)
-                close ();
-            break;
-    }
-
-    own_t::process_term (linger_);
-}
-
 void zmq::socks_connecter_t::in_event ()
 {
-    zmq_assert (_status != unplugged && _status != waiting_for_reconnect_time);
+    zmq_assert (_status != unplugged);
 
     if (_status == waiting_for_choice) {
         int rc = _choice_decoder.input (_s);
@@ -127,7 +86,7 @@ void zmq::socks_connecter_t::in_event ()
             if (rc == -1)
                 error ();
             else {
-                std::string hostname = "";
+                std::string hostname;
                 uint16_t port = 0;
                 if (parse_address (_addr->address, hostname, port) == -1)
                     error ();
@@ -150,22 +109,11 @@ void zmq::socks_connecter_t::in_event ()
             if (rc == -1)
                 error ();
             else {
-                //  Create the engine object for this connection.
-                stream_engine_t *engine =
-                  new (std::nothrow) stream_engine_t (_s, options, _endpoint);
-                alloc_assert (engine);
-
-                //  Attach the engine to the corresponding session object.
-                send_attach (_session, engine);
-
-                _socket->event_connected (_endpoint, _s);
-
-                rm_fd (_handle);
+                rm_handle ();
+                create_engine (
+                  _s, get_socket_name<tcp_address_t> (_s, socket_end_local));
                 _s = -1;
                 _status = unplugged;
-
-                //  Shut the connecter down.
-                terminate ();
             }
         }
     } else
@@ -209,8 +157,10 @@ void zmq::socks_connecter_t::out_event ()
     }
 }
 
-void zmq::socks_connecter_t::initiate_connect ()
+void zmq::socks_connecter_t::start_connecting ()
 {
+    zmq_assert (_status == unplugged);
+
     //  Open the connecting socket.
     const int rc = connect_to_proxy ();
 
@@ -225,13 +175,14 @@ void zmq::socks_connecter_t::initiate_connect ()
         _handle = add_fd (_s);
         set_pollout (_handle);
         _status = waiting_for_proxy_connection;
-        _socket->event_connect_delayed (_endpoint, zmq_errno ());
+        _socket->event_connect_delayed (
+          make_unconnected_connect_endpoint_pair (_endpoint), zmq_errno ());
     }
     //  Handle any other error condition by eventual reconnect.
     else {
         if (_s != retired_fd)
             close ();
-        start_timer ();
+        add_reconnect_timer ();
     }
 }
 
@@ -248,13 +199,6 @@ int zmq::socks_connecter_t::process_server_response (
     return response_.response_code == 0 ? 0 : -1;
 }
 
-void zmq::socks_connecter_t::timer_event (int id_)
-{
-    zmq_assert (_status == waiting_for_reconnect_time);
-    zmq_assert (id_ == reconnect_timer_id);
-    initiate_connect ();
-}
-
 void zmq::socks_connecter_t::error ()
 {
     rm_fd (_handle);
@@ -263,31 +207,7 @@ void zmq::socks_connecter_t::error ()
     _choice_decoder.reset ();
     _request_encoder.reset ();
     _response_decoder.reset ();
-    start_timer ();
-}
-
-void zmq::socks_connecter_t::start_timer ()
-{
-    const int interval = get_new_reconnect_ivl ();
-    add_timer (interval, reconnect_timer_id);
-    _status = waiting_for_reconnect_time;
-    _socket->event_connect_retried (_endpoint, interval);
-}
-
-int zmq::socks_connecter_t::get_new_reconnect_ivl ()
-{
-    //  The new interval is the current interval + random value.
-    const int interval =
-      _current_reconnect_ivl + generate_random () % options.reconnect_ivl;
-
-    //  Only change the current reconnect interval  if the maximum reconnect
-    //  interval was set and if it's larger than the reconnect interval.
-    if (options.reconnect_ivl_max > 0
-        && options.reconnect_ivl_max > options.reconnect_ivl)
-        //  Calculate the next interval
-        _current_reconnect_ivl =
-          std::min (_current_reconnect_ivl * 2, options.reconnect_ivl_max);
-    return interval;
+    add_reconnect_timer ();
 }
 
 int zmq::socks_connecter_t::connect_to_proxy ()
@@ -295,49 +215,31 @@ int zmq::socks_connecter_t::connect_to_proxy ()
     zmq_assert (_s == retired_fd);
 
     //  Resolve the address
-    LIBZMQ_DELETE (_proxy_addr->resolved.tcp_addr);
-    _proxy_addr->resolved.tcp_addr = new (std::nothrow) tcp_address_t ();
-    alloc_assert (_proxy_addr->resolved.tcp_addr);
+    if (_addr->resolved.tcp_addr != NULL) {
+        LIBZMQ_DELETE (_addr->resolved.tcp_addr);
+    }
 
-    int rc = _proxy_addr->resolved.tcp_addr->resolve (
-      _proxy_addr->address.c_str (), false, options.ipv6);
-    if (rc != 0) {
-        LIBZMQ_DELETE (_proxy_addr->resolved.tcp_addr);
+    _addr->resolved.tcp_addr = new (std::nothrow) tcp_address_t ();
+    alloc_assert (_addr->resolved.tcp_addr);
+    //  Automatic fallback to ipv4 is disabled here since this was the existing
+    //  behaviour, however I don't see a real reason for this. Maybe this can
+    //  be changed to true (and then the parameter can be removed entirely).
+    _s = tcp_open_socket (_addr->address.c_str (), options, false, false,
+                          _addr->resolved.tcp_addr);
+    if (_s == retired_fd) {
+        //  TODO we should emit some event in this case!
+
+        LIBZMQ_DELETE (_addr->resolved.tcp_addr);
         return -1;
     }
-    zmq_assert (_proxy_addr->resolved.tcp_addr != NULL);
-    const tcp_address_t *tcp_addr = _proxy_addr->resolved.tcp_addr;
-
-    //  Create the socket.
-    _s = open_socket (tcp_addr->family (), SOCK_STREAM, IPPROTO_TCP);
-    if (_s == retired_fd)
-        return -1;
-
-    //  On some systems, IPv4 mapping in IPv6 sockets is disabled by default.
-    //  Switch it on in such cases.
-    if (tcp_addr->family () == AF_INET6)
-        enable_ipv4_mapping (_s);
-
-    // Set the IP Type-Of-Service priority for this socket
-    if (options.tos != 0)
-        set_ip_type_of_service (_s, options.tos);
-
-    // Bind the socket to a device if applicable
-    if (!options.bound_device.empty ())
-        bind_to_device (_s, options.bound_device);
+    zmq_assert (_addr->resolved.tcp_addr != NULL);
 
     // Set the socket to non-blocking mode so that we get async connect().
     unblock_socket (_s);
 
-    //  Set the socket buffer limits for the underlying socket.
-    if (options.sndbuf >= 0)
-        set_tcp_send_buffer (_s, options.sndbuf);
-    if (options.rcvbuf >= 0)
-        set_tcp_receive_buffer (_s, options.rcvbuf);
+    const tcp_address_t *const tcp_addr = _addr->resolved.tcp_addr;
 
-    // Set the IP Type-Of-Service for the underlying socket
-    if (options.tos != 0)
-        set_ip_type_of_service (_s, options.tos);
+    int rc;
 
     // Set a source address for conversations
     if (tcp_addr->has_src_addr ()) {
@@ -431,20 +333,6 @@ zmq::fd_t zmq::socks_connecter_t::check_proxy_connection ()
     return 0;
 }
 
-void zmq::socks_connecter_t::close ()
-{
-    zmq_assert (_s != retired_fd);
-#ifdef ZMQ_HAVE_WINDOWS
-    const int rc = closesocket (_s);
-    wsa_assert (rc != SOCKET_ERROR);
-#else
-    const int rc = ::close (_s);
-    errno_assert (rc == 0);
-#endif
-    _socket->event_closed (_endpoint, _s);
-    _s = retired_fd;
-}
-
 int zmq::socks_connecter_t::parse_address (const std::string &address_,
                                            std::string &hostname_,
                                            uint16_t &port_)
